@@ -136,12 +136,12 @@ void PrinterManager::getRemotePrinters(const QString &uri, const QString &uriSch
                           KCUPS_PRINTER_INFO,
                           KCUPS_PRINTER_MAKE_AND_MODEL});
 
-    request->waitTillFinished();
-
-    if (request) {
-        const auto printers = request->printers();
-        if (request->hasError()) {
-            Q_EMIT requestError(request->errorMsg());
+    // If the request invoke fails or the actual request fails, we will get
+    // a "finished" signal
+    connect(request, &KCupsRequest::finished, this, [this, conn](KCupsRequest *req) {
+        const auto printers = req->printers();
+        if (req->hasError()) {
+            Q_EMIT requestError(req->errorMsg());
         } else {
             for (const auto &p : printers) {
                 const auto mm = p.makeAndModel();
@@ -163,10 +163,9 @@ void PrinterManager::getRemotePrinters(const QString &uri, const QString &uriSch
             Q_EMIT remotePrintersLoaded();
         }
 
-        request->deleteLater();
-    }
-
-    conn->deleteLater();
+        req->deleteLater();
+        conn->deleteLater();
+    });
 }
 
 void PrinterManager::clearRemotePrinters()
@@ -203,6 +202,7 @@ void PrinterManager::savePrinter(const QString &name, const QVariantMap &saveArg
     const bool addMode = args.take(u"add"_s).toBool();
     // Will only be set if default is changed to true
     const bool isDefault = args.take(u"isDefault"_s).toBool();
+    bool addModify = false;
 
     if (addMode) {
         args[KCUPS_PRINTER_STATE] = IPP_PRINTER_IDLE;
@@ -212,17 +212,7 @@ void PrinterManager::savePrinter(const QString &name, const QVariantMap &saveArg
                    << fileName << Qt::endl
                    << "args" << args;
 
-    QPointer<KCupsRequest> request = new KCupsRequest;
-
-    /** Default printer is handled by CUPS independently of the other printer
-     * attributes. If changing an existing printer and "default" is set
-     * then save immediately
-     */
-    if (!addMode && isDefault) {
-        request->setDefaultPrinter(name);
-        request->waitTillFinished();
-    }
-
+    auto request = new KCupsRequest;
     if (isClass) {
         // Member list is a QVariantList, kcupslib wants to see
         // a QStringList
@@ -232,123 +222,136 @@ void PrinterManager::savePrinter(const QString &name, const QVariantMap &saveArg
         }
         if (!args.isEmpty()) {
             request->addOrModifyClass(name, args);
-            request->waitTillFinished();
+            addModify = true;
         }
     } else {
         if (!args.isEmpty() || !fileName.isEmpty()) {
             request->addOrModifyPrinter(name, args, fileName);
-            request->waitTillFinished();
+            addModify = true;
         }
     }
 
-    // If adding a new printer and it's set as default, save explicitly
-    if (addMode && isDefault) {
-        request->setDefaultPrinter(name);
-        request->waitTillFinished();
-    }
-
-    if (request) {
-        if (!request->hasError()) {
-            Q_EMIT saveDone();
-        } else {
-            Q_EMIT requestError((isClass ? i18nc("@info", "Failed to configure class: ") : i18nc("@info", "Failed to configure printer: "))
-                                + request->errorMsg());
-            qCWarning(PMKCM) << "Failed to save printer/class" << name << request->errorMsg();
+    /** If no other printer attrs are changed, we still have to check default printer
+     * Default printer is handled by CUPS independently of the other printer
+     * attributes. if Default is set save explicitly.
+     */
+    if (!addModify) {
+        if (isDefault) {
+            qCDebug(PMKCM) << "Saving printer DEFAULT:" << name;
+            request->setDefaultPrinter(name);
         }
         request->deleteLater();
+        Q_EMIT saveDone();
+    } else {
+        connect(request, &KCupsRequest::finished, this, [this, isClass, name, isDefault](KCupsRequest *req) {
+            if (!req->hasError()) {
+                // Also check default printer flag
+                if (isDefault) {
+                    qCDebug(PMKCM) << "Saving printer DEFAULT:" << name;
+                    auto r = setupRequest([this]() {
+                        Q_EMIT saveDone();
+                    });
+                    r->setDefaultPrinter(name);
+                } else {
+                    Q_EMIT saveDone();
+                }
+            } else {
+                Q_EMIT requestError((isClass ? i18nc("@info", "Failed to configure class: ") : i18nc("@info", "Failed to configure printer: "))
+                                    + req->errorMsg());
+                qCWarning(PMKCM) << "Failed to save printer/class" << name << req->errorMsg();
+            }
+            req->deleteLater();
+        });
     }
 }
 
-QVariantMap PrinterManager::getPrinterPPD(const QString &name)
+void PrinterManager::loadPrinterPPD(const QString &name)
 {
-    QPointer<KCupsRequest> request = new KCupsRequest;
+    auto request = new KCupsRequest;
     request->getPrinterPPD(name);
-    request->waitTillFinished();
-    if (!request) {
-        return {};
-    }
+    connect(request, &KCupsRequest::finished, this, [this](KCupsRequest *req) {
+        const auto filename = req->printerPPD();
+        const auto err = req->errorMsg();
+        req->deleteLater();
 
-    const auto filename = request->printerPPD();
-    const auto err = request->errorMsg();
-    request->deleteLater();
+        ppd_file_t *ppd = nullptr;
+        if (!filename.isEmpty()) {
+            ppd = ppdOpenFile(qUtf8Printable(filename));
+            unlink(qUtf8Printable(filename));
+        }
 
-    ppd_file_t *ppd = nullptr;
-    if (!filename.isEmpty()) {
-        ppd = ppdOpenFile(qUtf8Printable(filename));
-        unlink(qUtf8Printable(filename));
-    }
+        if (ppd == nullptr) {
+            qCWarning(PMKCM) << "Could not open ppd file:" << filename << err;
+            Q_EMIT ppdLoaded({});
+            return;
+        }
 
-    if (ppd == nullptr) {
-        qCWarning(PMKCM) << "Could not open ppd file:" << filename << err;
-        return {};
-    }
+        ppdLocalize(ppd);
+        // select the default options on the ppd file
+        ppdMarkDefaults(ppd);
 
-    ppdLocalize(ppd);
-    // select the default options on the ppd file
-    ppdMarkDefaults(ppd);
+        const char *lang_encoding;
+        lang_encoding = ppd->lang_encoding;
+        QStringDecoder codec;
+        if (lang_encoding && !strcasecmp(lang_encoding, "UTF-8")) {
+            codec = QStringDecoder(QStringDecoder::Utf8);
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin1")) {
+            codec = QStringDecoder(QStringDecoder::Latin1);
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin2")) {
+            codec = QStringDecoder("ISO-8859-2");
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin5")) {
+            codec = QStringDecoder("ISO-8859-5");
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "JIS83-RKSJ")) {
+            codec = QStringDecoder("SHIFT-JIS");
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "MacStandard")) {
+            codec = QStringDecoder("MACINTOSH");
+        } else if (lang_encoding && !strcasecmp(lang_encoding, "WindowsANSI")) {
+            codec = QStringDecoder("WINDOWS-1252");
+        } else {
+            qCWarning(PMKCM) << "Unknown ENCODING:" << lang_encoding;
+            codec = QStringDecoder(lang_encoding);
+        }
+        // Fallback
+        if (!codec.isValid()) {
+            codec = QStringDecoder(QStringDecoder::Utf8);
+        }
 
-    const char *lang_encoding;
-    lang_encoding = ppd->lang_encoding;
-    QStringDecoder codec;
+        qCDebug(PMKCM) << codec(ppd->pcfilename) << codec(ppd->modelname) << codec(ppd->shortnickname);
 
-    if (lang_encoding && !strcasecmp(lang_encoding, "UTF-8")) {
-        codec = QStringDecoder(QStringDecoder::Utf8);
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin1")) {
-        codec = QStringDecoder(QStringDecoder::Latin1);
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin2")) {
-        codec = QStringDecoder("ISO-8859-2");
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "ISOLatin5")) {
-        codec = QStringDecoder("ISO-8859-5");
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "JIS83-RKSJ")) {
-        codec = QStringDecoder("SHIFT-JIS");
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "MacStandard")) {
-        codec = QStringDecoder("MACINTOSH");
-    } else if (lang_encoding && !strcasecmp(lang_encoding, "WindowsANSI")) {
-        codec = QStringDecoder("WINDOWS-1252");
-    } else {
-        qCWarning(PMKCM) << "Unknown ENCODING:" << lang_encoding;
-        codec = QStringDecoder(lang_encoding);
-    }
-    // Fallback
-    if (!codec.isValid()) {
-        codec = QStringDecoder(QStringDecoder::Utf8);
-    }
+        QString make, makeAndModel, file;
+        if (ppd->manufacturer) {
+            make = codec(ppd->manufacturer);
+        }
 
-    qCDebug(PMKCM) << codec(ppd->pcfilename) << codec(ppd->modelname) << codec(ppd->shortnickname);
+        if (ppd->nickname) {
+            makeAndModel = codec(ppd->nickname);
+        }
 
-    QString make, makeAndModel, file;
-    if (ppd->manufacturer) {
-        make = codec(ppd->manufacturer);
-    }
+        if (ppd->pcfilename) {
+            file = codec(ppd->pcfilename);
+        }
 
-    if (ppd->nickname) {
-        makeAndModel = codec(ppd->nickname);
-    }
-
-    if (ppd->pcfilename) {
-        file = codec(ppd->pcfilename);
-    }
-
-    ppd_attr_t *ppdattr;
-    bool autoConfig = false;
-    if (ppd->num_filters == 0
-        || ((ppdattr = ppdFindAttr(ppd, "cupsCommands", nullptr)) != nullptr && ppdattr->value && strstr(ppdattr->value, "AutoConfigure"))) {
-        autoConfig = true;
-    } else {
-        for (int i = 0; i < ppd->num_filters; ++i) {
-            if (!strncmp(ppd->filters[i], "application/vnd.cups-postscript", 31)) {
-                autoConfig = true;
-                break;
+        ppd_attr_t *ppdattr;
+        bool autoConfig = false;
+        if (ppd->num_filters == 0
+            || ((ppdattr = ppdFindAttr(ppd, "cupsCommands", nullptr)) != nullptr && ppdattr->value && strstr(ppdattr->value, "AutoConfigure"))) {
+            autoConfig = true;
+        } else {
+            for (int i = 0; i < ppd->num_filters; ++i) {
+                if (!strncmp(ppd->filters[i], "application/vnd.cups-postscript", 31)) {
+                    autoConfig = true;
+                    break;
+                }
             }
         }
-    }
 
-    return {{u"autoConfig"_s, autoConfig},
-            {u"file"_s, QString()},
-            {u"pcfile"_s, file},
-            {u"type"_s, QVariant(PMTypes::PPDType::Custom)},
-            {u"make"_s, make},
-            {u"makeModel"_s, makeAndModel}};
+        Q_EMIT ppdLoaded({{u"autoConfig"_s, autoConfig},
+                          {u"file"_s, QString()},
+                          {u"pcfile"_s, file},
+                          {u"type"_s, QVariant(PMTypes::PPDType::Custom)},
+                          {u"make"_s, make},
+                          {u"makeModel"_s, makeAndModel}});
+    });
 }
 
 void PrinterManager::pausePrinter(const QString &name)
@@ -369,15 +372,13 @@ bool PrinterManager::isIPPCapable(const QString &uri)
      * Per CUPS
      * "dnssd:" URIs with "._ipp._tcp" or "._ipps._tcp"
      *  or "ipp:" or "ipps:" URIs can be of a driverless printer
-    */
+     */
     if (uri.startsWith(u"dnssd:"_s, Qt::CaseInsensitive)
-        && (uri.contains(u"._ipp._tcp"_s, Qt::CaseInsensitive)
-            || uri.contains(u"._ipps._tcp"_s, Qt::CaseInsensitive))) {
+        && (uri.contains(u"._ipp._tcp"_s, Qt::CaseInsensitive) || uri.contains(u"._ipps._tcp"_s, Qt::CaseInsensitive))) {
         return true;
     }
 
-    if (uri.startsWith(u"ipp:"_s, Qt::CaseInsensitive)
-            || uri.startsWith(u"ipps:"_s, Qt::CaseInsensitive)) {
+    if (uri.startsWith(u"ipp:"_s, Qt::CaseInsensitive) || uri.startsWith(u"ipps:"_s, Qt::CaseInsensitive)) {
         return true;
     }
 
@@ -390,7 +391,10 @@ void PrinterManager::getRecommendedDrivers(const QString &deviceId, const QStrin
 
     m_recommendedDrivers.clear();
 
-    auto call = QDBusMessage::createMethodCall(u"org.fedoraproject.Config.Printing"_s, u"/org/fedoraproject/Config/Printing"_s, u"org.fedoraproject.Config.Printing"_s, u"GetBestDrivers"_s);
+    auto call = QDBusMessage::createMethodCall(u"org.fedoraproject.Config.Printing"_s,
+                                               u"/org/fedoraproject/Config/Printing"_s,
+                                               u"org.fedoraproject.Config.Printing"_s,
+                                               u"GetBestDrivers"_s);
     call.setArguments({deviceId, makeAndModel, deviceUri});
     const auto pending = QDBusConnection::sessionBus().asyncCall(call);
     const auto watcher = new QDBusPendingCallWatcher(pending, this);
@@ -499,24 +503,26 @@ void PrinterManager::saveServerSettings(const QVariantMap &settings)
     /**
      * The CUPS server will stop/start when settings change.  This will either be a stop signal
      * from CUPS and/or the non-fatal error below
-    */
+     */
     auto request = new KCupsRequest;
     request->setServerSettings(server);
-    request->waitTillFinished();
+    // If the request invoke fails or the actual request fails, we will get
+    // a "finished" signal
+    connect(request, &KCupsRequest::finished, this, [this, settings](KCupsRequest *req) {
+        if (req->error() == IPP_AUTHENTICATION_CANCELED) {
+            // If the user has cancelled auth
+            Q_EMIT requestError(i18nc("@info", "Server Settings Not Saved: (%1): %2", req->serverError(), req->errorMsg()));
+        } else if (req->error() == IPP_SERVICE_UNAVAILABLE || req->error() == IPP_INTERNAL_ERROR) {
+            // this is the expected "error" after settings have changed as the server is restarting
+            m_serverSettings = settings;
+            Q_EMIT serverSettingsChanged();
+            qCDebug(PMKCM) << "CUPS SETTINGS SAVED!" << settings;
+        } else {
+            Q_EMIT requestError(i18nc("@info", "Fatal Save Server Settings: (%1): %2", req->serverError(), req->errorMsg()));
+        }
 
-    if (request->error() == IPP_AUTHENTICATION_CANCELED) {
-        // If the user has cancelled auth
-        Q_EMIT requestError(i18nc("@info", "Server Settings Not Saved: (%1): %2", request->serverError(), request->errorMsg()));
-    } else if (request->error() == IPP_SERVICE_UNAVAILABLE || request->error() == IPP_INTERNAL_ERROR) {
-        // this is the expected "error" after settings have changed as the server is restarting
-        m_serverSettings = settings;
-        Q_EMIT serverSettingsChanged();
-        qCDebug(PMKCM) << "CUPS SETTINGS SAVED!" << settings;
-    } else {
-        Q_EMIT requestError(i18nc("@info", "Fatal Save Server Settings: (%1): %2", request->serverError(), request->errorMsg()));
-    }
-
-    request->deleteLater();
+        req->deleteLater();
+    });
 }
 
 bool PrinterManager::shareConnectedPrinters() const
