@@ -91,11 +91,6 @@ PrinterModel::PrinterModel(QObject *parent)
     connect(KCupsConnection::global(), &KCupsConnection::serverStopped, this, &PrinterModel::serverChanged);
     connect(KCupsConnection::global(), &KCupsConnection::serverRestarted, this, &PrinterModel::serverChanged);
 
-    // Deprecated stuff that works better than the above
-    connect(KCupsConnection::global(), &KCupsConnection::rhPrinterAdded, this, &PrinterModel::insertUpdatePrinterName);
-    connect(KCupsConnection::global(), &KCupsConnection::rhPrinterRemoved, this, &PrinterModel::printerRemovedName);
-    connect(KCupsConnection::global(), &KCupsConnection::rhQueueChanged, this, &PrinterModel::insertUpdatePrinterName);
-
     connect(this, &PrinterModel::rowsInserted, this, &PrinterModel::slotCountChanged);
     connect(this, &PrinterModel::rowsRemoved, this, &PrinterModel::slotCountChanged);
     connect(this, &PrinterModel::modelReset, this, &PrinterModel::slotCountChanged);
@@ -150,19 +145,25 @@ void PrinterModel::getDestsFinished(KCupsRequest *request)
             removeRow(rowCount() - 1);
         }
 
-        setDisplayLocationHint();
+        updateDisplayHints();
 
         Q_EMIT error(IPP_OK, QString(), QString());
     }
     request->deleteLater();
 }
 
-void PrinterModel::setDisplayLocationHint()
+void PrinterModel::updateDisplayHints()
 {
     QStringList locList;
+    bool printersOnly = true;
 
-    // get location list
     for (int i = 0; i < rowCount(); i++) {
+        // Printers Only hint
+        if (item(i)->data(DestIsClass).toBool()) {
+            printersOnly = false;
+        }
+
+        // Location list hint
         const auto val = item(i)->data(DestLocation).toString();
         if (!val.isEmpty()) {
             locList.append(val);
@@ -171,23 +172,26 @@ void PrinterModel::setDisplayLocationHint()
     // only show the location if there is more than one printer
     // and at least two distinct locations exist
     locList.removeDuplicates();
-    m_displayLocationHint = rowCount() > 1 && locList.count() > 1;
-    Q_EMIT displayLocationHintChanged();
-}
+    bool displayLocationHint = rowCount() > 1 && locList.count() > 1;
+    if (m_showLocations != displayLocationHint) {
+        m_showLocations = displayLocationHint;
+        Q_EMIT showLocationsChanged();
+    }
 
-bool PrinterModel::printersOnly() const
-{
-    for (int i = 0; i < rowCount(); i++) {
-        if (item(i)->data(DestIsClass).toBool()) {
-            return false;
-        }
+    if (m_hasOnlyPrinters != printersOnly) {
+        m_hasOnlyPrinters = printersOnly;
+        Q_EMIT hasOnlyPrintersChanged();
     }
-    return true;
 }
 
-bool PrinterModel::displayLocationHint() const
+bool PrinterModel::hasOnlyPrinters() const
 {
-    return m_displayLocationHint;
+    return m_hasOnlyPrinters;
+}
+
+bool PrinterModel::showLocations() const
+{
+    return m_showLocations;
 }
 
 void PrinterModel::slotCountChanged()
@@ -451,15 +455,6 @@ Qt::ItemFlags PrinterModel::flags(const QModelIndex &index) const
     return Qt::ItemIsSelectable | Qt::ItemIsEnabled;
 }
 
-void PrinterModel::insertUpdatePrinterName(const QString &printerName)
-{
-    auto request = new KCupsRequest;
-    connect(request, &KCupsRequest::finished, this, &PrinterModel::insertUpdatePrinterFinished);
-    // TODO how do we know if it's a class if this DBus signal
-    // does not tell us
-    request->getPrinterAttributes(printerName, false, m_attrs);
-}
-
 void PrinterModel::insertUpdatePrinter(const QString &text,
                                        const QString &printerUri,
                                        const QString &printerName,
@@ -474,7 +469,9 @@ void PrinterModel::insertUpdatePrinter(const QString &text,
     Q_UNUSED(printerIsAcceptingJobs)
 
     qCDebug(LIBKCUPS) << text << printerUri << printerName << printerState << printerStateReasons << printerIsAcceptingJobs;
-    insertUpdatePrinterName(printerName);
+    auto request = new KCupsRequest;
+    connect(request, &KCupsRequest::finished, this, &PrinterModel::insertUpdatePrinterFinished);
+    request->getPrinterAttributes(printerName, false, m_attrs);
 }
 
 void PrinterModel::insertUpdatePrinterFinished(KCupsRequest *request)
@@ -494,22 +491,10 @@ void PrinterModel::insertUpdatePrinterFinished(KCupsRequest *request)
             }
         }
     }
-    setDisplayLocationHint();
+    updateDisplayHints();
     request->deleteLater();
 }
 
-void PrinterModel::printerRemovedName(const QString &printerName)
-{
-    qCDebug(LIBKCUPS) << printerName;
-
-    // Look for the removed printer
-    int dest_row = destRow(printerName);
-    if (dest_row != -1) {
-        removeRows(dest_row, 1);
-    }
-    setDisplayLocationHint();
-}
-
 void PrinterModel::printerRemoved(const QString &text,
                                   const QString &printerUri,
                                   const QString &printerName,
@@ -530,7 +515,7 @@ void PrinterModel::printerRemoved(const QString &text,
     if (dest_row != -1) {
         removeRows(dest_row, 1);
     }
-    setDisplayLocationHint();
+    updateDisplayHints();
 }
 
 void PrinterModel::printerStateChanged(const QString &text,
@@ -580,7 +565,7 @@ void PrinterModel::printerModified(const QString &text,
                                    bool printerIsAcceptingJobs)
 {
     qCDebug(LIBKCUPS) << text << printerUri << printerName << printerState << printerStateReasons << printerIsAcceptingJobs;
-    setDisplayLocationHint();
+    updateDisplayHints();
 }
 
 void PrinterModel::serverChanged(const QString &text)
